# coding: utf-8
"""
Configuration variables.

"""

from __future__ import unicode_literals

pub_key = None
secret = None

api_version = '0.2'
api_base = 'https://api.uploadcare.com/'
upload_base = 'https://upload.uploadcare.com/'
cdn_base = 'https://ucarecdn.com/'

verify_api_ssl = True
verify_upload_ssl = True
